/*++
  
  Copyright (c) 1996 Microsoft Corporation.

  Copyright (c) 1995 Intel Corp

  Copyright (c) 1996 Bob Quinn
  
  Module Name:
  
    bsd_func.cpp
  
  Abstract:
  
    Contains handler functions for all BSD sockets compatible
	function hooks in the WinSock 2 debug trace DLL (dt_dll.dll).
  
--*/

#include "nowarn.h"  /* turn off benign warnings */
#ifndef _WINSOCKAPI_
#define _WINSOCKAPI_   /* Prevent inclusion of winsock.h in windows.h */
#endif
#include <windows.h>
#include "nowarn.h"  /* some warnings may have been turned back on */
#include <ws2spi.h>

#include "handlers.h"
#include "dt_dll.h"
#include "dt.h"

/*
 * Option flags per-socket.
 */
extern char FAR *sock_opts1[];
#define MAX_OPTS1  9
extern char FAR *sock_opts2[];
#define MAX_OPTS2  9
extern char FAR *sock_opts3[];
#define MAX_OPTS3  5
extern char sock_nodelay[];
extern char sock_pvd_cfg[];
extern char sock_optx[];

/*
 * Commands for ioctlsocket(),
 */
char *ioctl_ioc[] = {
	"IOC_VOID",		/* 0x2000000 */
	"IOC_OUT",		/* 0x4000000 */
	"IOC_IN",		/* 0x8000000 */
	"IOC_INOUT"		/* 0xC000000 */
};

char *ioctl_fio[] = {
	"FIONASYNC",	/* 0xd */
	"FIONBIO",		/* 0xe */
	"FIONREAD"		/* 0xf */
};

char *ioctl_sio[] = {
	"SIOCSHIWAT",	/* 0 */	
	"SIOCGHIWAT",	/* 1 */
	"SIOCSLOWAT",	/* 2 */
	"SIOCGLOWAT",	/* 3 */
	"unknown","unknown","unknown",
	"SIOCATMARK"	/* 7 */
};
/*++
  
  Note:

      This comment applies to all DTHandler_xxxx functions.
  
  Function Description:
  
      For the present, each handler function simply adds "xxxx
      called." to the end of the string passed in, and calls
      DTTextOut to output the whole string to the Debug Window.  This
      allows handler functions to be disabled easily, or to output
      more information (like the arguments) via multiple calls to
      DTTextOut, etc.  See the Debug/Trace documentation for more
      information. 
  
  Arguments:
  
      vl -- Variable used to strip arguments off the stack of 
      WSA[Pre|Post]ApiNotify, the calling function.

      ReturnValue -- A void pointer to the return value for the
      original API/SPI function.  Use a local variable to cast it to
      the appropriate type.

      LibraryName -- For API functions, this should just be
      "WinSock2", but for SPI functions it is the name of the service
      provider.  This is to distinguish among the multiple service
      providers that WinSock2 may be calling.

      Buffer -- A string containg some info about the thread and
      function call # of this call.  

      Index -- How many characters are in the above.
      
      BufLen -- How big is the buffer.

      PreOrPost -- TRUE if called by a WSAPreApiNotify, FALSE if
      called by WSAPostApiNotify.  
        
  Return Value:
  
      If the PreOrPost argument is FALSE, then the return value is
      ignored.  If it's TRUE, then the return value indicates to
      WSAPreApiNotify whether or not we want to short-circuit the API
      or SPI function.  TRUE means short-circuit it, FALSE mean don't.
  
--*/


BOOL CALLBACK
DTHandler_accept(
    IN     va_list vl,
    IN OUT LPVOID  ReturnValue,
    IN     LPSTR   LibraryName,
    OUT    char    *Buffer,
    IN     int     Index,
    IN     int     BufLen,
    IN     BOOL    PreOrPost)
{
    SOCKET *RetVal = (SOCKET *)ReturnValue;
    SOCKET *s = va_arg(vl, SOCKET *);
    struct sockaddr FAR **addr = va_arg(vl, struct sockaddr FAR **);
    int FAR **addrlen = va_arg(vl, int FAR **);

    int off = Index;
	if (PreOrPost) {
		off += wsprintf(Buffer + off, "-> accept()\r\n\t");
	} else {
		wsprintf(Buffer + off, "<- accept() returned: %d\r\n\t",*RetVal);
	}

	off += wsprintf(Buffer + off, "socket = %d\r\n",*s);
	if (*addr) {
		off = FormatSockaddrIn((LPSOCKADDR_IN)*addr, Buffer, off);
	} else {
		off += wsprintf(Buffer + off,"name is NULL\r\n");
	}
	if (*addrlen) {
		wsprintf (Buffer + off, "addrlen: %d\r\n", **addrlen);
	} else {
		off += wsprintf(Buffer + off,"addrlen is NULL\r\n");
	}

    DTTextOut(DebugWindow, LogFileHandle, Buffer, OutputStyle);
    return(FALSE);
}


BOOL CALLBACK
DTHandler_bind(
    IN     va_list vl,
    IN OUT LPVOID  ReturnValue,
    IN     LPSTR   LibraryName,
    OUT    char    *Buffer,
    IN     int     Index,
    IN     int     BufLen,
    IN     BOOL    PreOrPost)
{
    int *RetVal = (int *)ReturnValue;
    SOCKET *s = va_arg(vl, SOCKET *);
    struct sockaddr FAR **name = va_arg(vl, struct sockaddr FAR **);
    int *namelen = va_arg(vl, int *);

    int off = Index;
	if (PreOrPost) {
		off += wsprintf(Buffer + off, "-> bind()\r\n\t");
	} else {
		off += wsprintf(Buffer + off, "<- bind() returned: %d\r\n\t",*RetVal);
	}
	off += wsprintf(Buffer + off, "socket = %d\r\n\t",*s);
	if (*name) {
		off = FormatSockaddrIn((LPSOCKADDR_IN)*name, Buffer, off);
	} else {
		off += wsprintf(Buffer + off,"name is NULL\r\n\t");
	}
	wsprintf (Buffer + off, "namelen: %d\r\n", *namelen);

    DTTextOut(DebugWindow, LogFileHandle, Buffer, OutputStyle);
    return(FALSE);
}


BOOL CALLBACK
DTHandler_closesocket(
    IN     va_list vl,
    IN OUT LPVOID  ReturnValue,
    IN     LPSTR   LibraryName,
    OUT    char    *Buffer,
    IN     int     Index,
    IN     int     BufLen,
    IN     BOOL    PreOrPost)
{
    int *RetVal = (int *)ReturnValue;
    SOCKET *s = va_arg(vl, SOCKET *);

    int off = Index;
	if (PreOrPost) {
		off += wsprintf(Buffer + off, "-> closesocket(socket = %d)\r\n", *s);
	} else {
		wsprintf(Buffer + off, "<- closesocket() returned: %d\r\n",
            *RetVal);
	}
    DTTextOut(DebugWindow, LogFileHandle, Buffer, OutputStyle);
    return(FALSE);
}


BOOL CALLBACK
DTHandler_connect(
    IN     va_list vl,
    IN OUT LPVOID  ReturnValue,
    IN     LPSTR   LibraryName,
    OUT    char    *Buffer,
    IN     int     Index,
    IN     int     BufLen,
    IN     BOOL    PreOrPost)
{
    int *RetVal = (int *)ReturnValue;
    SOCKET *s = va_arg(vl, SOCKET *);
    struct sockaddr FAR **name = va_arg(vl, struct sockaddr FAR **);
    int *namelen = va_arg(vl, int *);

    int off = Index;
	if (PreOrPost) {
		off += wsprintf(Buffer + off, "-> connect()\r\n\t");
		off += wsprintf(Buffer + off, "socket = %d\r\n\t",*s);
		if (*name) {
			off = FormatSockaddrIn((LPSOCKADDR_IN)*name, Buffer, off);
		} else {
			off += wsprintf(Buffer + off,"name is NULL\r\n\t");
		}
		wsprintf (Buffer + off, "namelen: %d\r\n", *namelen);
	} else {
		wsprintf(Buffer + off, "<- connect() returned: %d\r\n", *RetVal);
	}

    DTTextOut(DebugWindow, LogFileHandle, Buffer, OutputStyle);
    return(FALSE);
}


BOOL CALLBACK
DTHandler_getpeername(
    IN     va_list vl,
    IN OUT LPVOID  ReturnValue,
    IN     LPSTR   LibraryName,
    OUT    char    *Buffer,
    IN     int     Index,
    IN     int     BufLen,
    IN     BOOL    PreOrPost)
{
    int *RetVal = (int *)ReturnValue;
    SOCKET *s = va_arg(vl, SOCKET *);
    struct sockaddr FAR **name = va_arg(vl, struct sockaddr FAR **);
    int FAR **namelen = va_arg(vl, int FAR **);

    int off = Index;
	if (PreOrPost) {
		off += wsprintf(Buffer + off, "-> getpeername()\r\n\t");
	} else {
		off += wsprintf(Buffer + off, "<- getpeername() returned: %d\r\n\t",*RetVal);
	}
	off += wsprintf(Buffer + off, "socket = %d\r\n\t",*s);
	if (*name) {
		off = FormatSockaddrIn((LPSOCKADDR_IN)*name, Buffer, off);
	} else {
		off += wsprintf(Buffer + off,"name is NULL\r\n\t");
	}
	if (*namelen) {
		wsprintf (Buffer + off, "namelen: %d\r\n", **namelen);
	} else {
		wsprintf (Buffer + off, "namelen is NULL\r\n");
	}

    DTTextOut(DebugWindow, LogFileHandle, Buffer, OutputStyle);
    return(FALSE);
}


BOOL CALLBACK
DTHandler_getsockname(
    IN     va_list vl,
    IN OUT LPVOID  ReturnValue,
    IN     LPSTR   LibraryName,
    OUT    char    *Buffer,
    IN     int     Index,
    IN     int     BufLen,
    IN     BOOL    PreOrPost)
{
    int *RetVal = (int *)ReturnValue;
    SOCKET *s = va_arg(vl, SOCKET *);
    struct sockaddr FAR **name = va_arg(vl, struct sockaddr FAR **);
    int FAR **namelen = va_arg(vl, int FAR **);

    int off = Index;
	if (PreOrPost) {
		off += wsprintf(Buffer + off, "-> getsockname()\r\n\t");
	} else {
		off += wsprintf(Buffer + off, "<- getsockname() returned: %d\r\n\t",*RetVal);
	}
	off += wsprintf(Buffer + off, "socket = %d\r\n\t",*s);
	if (*name) {
		off = FormatSockaddrIn((LPSOCKADDR_IN)*name, Buffer, off);
	} else {
		off += wsprintf(Buffer + off,"name is NULL\r\n\t");
	}
	if (*namelen) {
		wsprintf (Buffer + off, "namelen: %d\r\n", **namelen);
	} else {
		wsprintf (Buffer + off, "namelen is NULL\r\n");
	}

    DTTextOut(DebugWindow, LogFileHandle, Buffer, OutputStyle);
    return(FALSE);
}


BOOL CALLBACK
DTHandler_getsockopt(
    IN     va_list vl,
    IN OUT LPVOID  ReturnValue,
    IN     LPSTR   LibraryName,
    OUT    char    *Buffer,
    IN     int     Index,
    IN     int     BufLen,
    IN     BOOL    PreOrPost)
{
    int *RetVal = (int *)ReturnValue;
    SOCKET *s = va_arg(vl, SOCKET *);
    int *level = va_arg(vl, int *);
    int *optname = va_arg(vl, int *);
    char FAR **optval = va_arg(vl, char FAR **);
    int FAR **optlen = va_arg(vl, int FAR **);

    int off = Index;
	if (PreOrPost) {
		off += wsprintf(Buffer + off, "-> getsockopt()\r\n\t");
	} else {
		off += wsprintf(Buffer + off, "<- getsockopt() returned: %d\r\n\t",*RetVal);
	}
	off += wsprintf(Buffer + off, "socket = %d\r\n\t",*s);
	off += wsprintf(Buffer + off, "level  = %d\r\n\t",*level);
	if (*optname) {
		int i, iOptGroup = *optname & 0xF000;
		LPSTR szOptString = 0L;

		for (i=0; i<10; i++) {
			int	iOptName  = (int) *optval;
			int iOptIndex = iOptName;
			
			if (iOptName == PVD_CONFIG) {
				szOptString = sock_pvd_cfg;
				break;
			}
			iOptIndex>>i; 
			if (iOptIndex & 0x0001) {
				if (!iOptGroup && (iOptIndex < MAX_OPTS1)) {
					if ((*level == IPPROTO_TCP) && (iOptName == TCP_NODELAY))
						szOptString = sock_nodelay;
					else
						szOptString = sock_opts1[i];
				} else if ((iOptGroup & 0x1000) && (iOptIndex < MAX_OPTS2)) {
					if (*optname == SO_DONTLINGER)
						szOptString = sock_opts2[8];
					else
						szOptString = sock_opts2[i];
				} else if ((iOptGroup & 0x2000) && (iOptIndex < MAX_OPTS3)) {
						szOptString = sock_opts3[i];
				}
				break;
			}
		}
		if (!szOptString)
			szOptString = sock_optx;	/* Unknown Option */ 
		off += wsprintf(Buffer + off,"optname: %s\r\n\t", szOptString);
	} else {
		off += wsprintf(Buffer + off,"optname is NULL\r\n\t");
	}
	if (*optval) {
		off += wsprintf(Buffer + off, "optval = (0x%08x)\r\n\t",*optval);
	} else {
		off += wsprintf(Buffer + off, "optval is NULL\r\n\t");
	}
	if (*optlen) {
		wsprintf (Buffer + off, "optlen: %d\r\n", **optlen);
	} else {
		wsprintf (Buffer + off, "optlen is NULL\r\n");
	}

    DTTextOut(DebugWindow, LogFileHandle, Buffer, OutputStyle);
    return(FALSE);
}


BOOL CALLBACK
DTHandler_htonl(
    IN     va_list vl,
    IN OUT LPVOID  ReturnValue,
    IN     LPSTR   LibraryName,
    OUT    char    *Buffer,
    IN     int     Index,
    IN     int     BufLen,
    IN     BOOL    PreOrPost)
{
    u_long *RetVal = (u_long *)ReturnValue;
    u_long *hostlong = va_arg(vl, u_long *);

	if (PreOrPost) {
		wsprintf(Buffer + Index, 
			"-> htonl() hostlong: %d (0x%lx)\r\n", *hostlong, *hostlong);
	} else {
		wsprintf(Buffer + Index,
			"<- htonl() returned: %d (0x%lx)\r\n", *RetVal, *RetVal);
	}
    DTTextOut(DebugWindow, LogFileHandle, Buffer, OutputStyle);
    return(FALSE);
}


BOOL CALLBACK
DTHandler_htons(
    IN     va_list vl,
    IN OUT LPVOID  ReturnValue,
    IN     LPSTR   LibraryName,
    OUT    char    *Buffer,
    IN     int     Index,
    IN     int     BufLen,
    IN     BOOL    PreOrPost)
{
    u_short *RetVal = (u_short *)ReturnValue;
    u_short *hostshort = va_arg(vl, u_short *);

	if (PreOrPost) {
		wsprintf(Buffer + Index, 
			"-> htons() hostshort: %d (0x%x)\r\n", *hostshort, *hostshort);
	} else {
		wsprintf(Buffer + Index,
			"<- htons() returned: %d (0x%x)\r\n", *RetVal, *RetVal);
	}
    DTTextOut(DebugWindow, LogFileHandle, Buffer, OutputStyle);
    return(FALSE);
}


BOOL CALLBACK
DTHandler_ioctlsocket(
    IN     va_list vl,
    IN OUT LPVOID  ReturnValue,
    IN     LPSTR   LibraryName,
    OUT    char    *Buffer,
    IN     int     Index,
    IN     int     BufLen,
    IN     BOOL    PreOrPost)
{
    int *RetVal = (int *)ReturnValue;
    SOCKET *s = va_arg(vl, SOCKET *);
    long *cmd = va_arg(vl, long *);
    u_long FAR **argp = va_arg(vl, u_long FAR **);

	int off = Index;
	LPSTR lpszCmd;
	u_long lCmd = (u_long)(*cmd & 0x0000000F);
	if (PreOrPost) {
		off += wsprintf(Buffer + off, 
			"-> ioctlsocket()\r\n\t");
	} else {
		off += wsprintf(Buffer + Index,
			"<- ioctlsocket() returned: %d (0x%lx)\r\n\t", *RetVal, *RetVal);
	}
	off += wsprintf(Buffer + off, "socket = %d\r\n\t",*s);

	if ((lCmd >= 0xd) && (lCmd <= 0xf)) {
		lpszCmd = ioctl_fio[lCmd-0xd];
	} else if (lCmd <= 7) {
		lpszCmd = ioctl_sio[lCmd];
	} else {
		lpszCmd = ioctl_sio[4]; // unknown cmd value
	}
	off += wsprintf(Buffer + off, "cmd  = %s (0x%08lx)\r\n\t",
		lpszCmd, *cmd);
	if (*argp) {
		wsprintf (Buffer + off, "argp: 0x%08lx\r\n", **argp);
	} else {
		wsprintf (Buffer + off, "argp is NULL\r\n");
	}
    DTTextOut(DebugWindow, LogFileHandle, Buffer, OutputStyle);
    return(FALSE);
}


BOOL CALLBACK
DTHandler_listen(
    IN     va_list vl,
    IN OUT LPVOID  ReturnValue,
    IN     LPSTR   LibraryName,
    OUT    char    *Buffer,
    IN     int     Index,
    IN     int     BufLen,
    IN     BOOL    PreOrPost)
{
    int *RetVal = (int *)ReturnValue;
    SOCKET *s = va_arg(vl, SOCKET *);
    int *backlog = va_arg(vl, int *);

    int off = Index;
	if (PreOrPost) {
		off += wsprintf(Buffer + off, "-> listen()\r\n\t");
		off += wsprintf(Buffer + off, "socket: %d\r\n\t", *s);
		wsprintf (Buffer + off, "backlog: %d\r\n", *backlog);
	} else {
		wsprintf(Buffer + off,
			"<- listen() returned: %d\r\n", *RetVal);
	}
    DTTextOut(DebugWindow, LogFileHandle, Buffer, OutputStyle);
    return(FALSE);
}


BOOL CALLBACK
DTHandler_ntohl(
    IN     va_list vl,
    IN OUT LPVOID  ReturnValue,
    IN     LPSTR   LibraryName,
    OUT    char    *Buffer,
    IN     int     Index,
    IN     int     BufLen,
    IN     BOOL    PreOrPost)
{
    u_long *RetVal = (u_long *)ReturnValue;
    u_long *netlong = va_arg(vl, u_long *);

	if (PreOrPost) {
		wsprintf(Buffer + Index, 
			"-> ntohl() netlong: %d (0x%lx)\r\n", *netlong, *netlong);
	} else {
		wsprintf(Buffer + Index,
			"<- ntohl() returned: %d (0x%lx)\r\n", *RetVal, *RetVal);
	}
    DTTextOut(DebugWindow, LogFileHandle, Buffer, OutputStyle);
    return(FALSE);
}


BOOL CALLBACK
DTHandler_ntohs(
    IN     va_list vl,
    IN OUT LPVOID  ReturnValue,
    IN     LPSTR   LibraryName,
    OUT    char    *Buffer,
    IN     int     Index,
    IN     int     BufLen,
    IN     BOOL    PreOrPost)
{
    u_short *RetVal = (u_short *)ReturnValue;
    u_short *netshort = va_arg(vl, u_short *);

	if (PreOrPost) {
		wsprintf(Buffer + Index, 
			"-> ntohs() netshort: %d (0x%x)\r\n", *netshort, *netshort);
	} else {
		wsprintf(Buffer + Index,
			"<- ntohs() returned: %d (0x%x)\r\n", *RetVal, *RetVal);
	}
    DTTextOut(DebugWindow, LogFileHandle, Buffer, OutputStyle);
    return(FALSE);
}


BOOL CALLBACK
DTHandler_recv(
    IN     va_list vl,
    IN OUT LPVOID  ReturnValue,
    IN     LPSTR   LibraryName,
    OUT    char    *Buffer,
    IN     int     Index,
    IN     int     BufLen,
    IN     BOOL    PreOrPost)
{
    int *RetVal = (int *)ReturnValue;
    SOCKET *s = va_arg(vl, SOCKET *);
    char FAR **buf = va_arg(vl, char FAR **);
    int *len = va_arg(vl, int *);
    int *flags = va_arg(vl, int *);

    wsprintf(Buffer + Index, "recv() %s.\r\n",
             PreOrPost ? "->" : "<-");
    DTTextOut(DebugWindow, LogFileHandle, Buffer, OutputStyle);
    return(FALSE);
}


BOOL CALLBACK
DTHandler_recvfrom(
    IN     va_list vl,
    IN OUT LPVOID  ReturnValue,
    IN     LPSTR   LibraryName,
    OUT    char    *Buffer,
    IN     int     Index,
    IN     int     BufLen,
    IN     BOOL    PreOrPost)
{
    int *RetVal = (int *)ReturnValue;
    SOCKET *s = va_arg(vl, SOCKET *);
    char FAR **buf = va_arg(vl, char FAR **);
    int *len = va_arg(vl, int *);
    int *flags = va_arg(vl, int *);
    struct sockaddr FAR **from = va_arg(vl, struct sockaddr FAR **);
    int FAR **fromlen = va_arg(vl, int FAR **);

    wsprintf(Buffer + Index, "recvfrom() %s.\r\n",
             PreOrPost ? "->" : "<-");
    DTTextOut(DebugWindow, LogFileHandle, Buffer, OutputStyle);
    return(FALSE);
}


BOOL CALLBACK
DTHandler_select(
    IN     va_list vl,
    IN OUT LPVOID  ReturnValue,
    IN     LPSTR   LibraryName,
    OUT    char    *Buffer,
    IN     int     Index,
    IN     int     BufLen,
    IN     BOOL    PreOrPost)
{
    int *RetVal = (int *)ReturnValue;
    int *nfds = va_arg(vl, int *);
    fd_set FAR **readfds = va_arg(vl, fd_set FAR **);
    fd_set FAR **writefds = va_arg(vl, fd_set FAR **);
    fd_set FAR **exceptfds = va_arg(vl, fd_set FAR **);
    struct timeval FAR **timeout = va_arg(vl, struct timeval FAR **);

    wsprintf(Buffer + Index, "select() %s.\r\n",
             PreOrPost ? "->" : "<-");
    DTTextOut(DebugWindow, LogFileHandle, Buffer, OutputStyle);
    return(FALSE);
}


BOOL CALLBACK
DTHandler_send(
    IN     va_list vl,
    IN OUT LPVOID  ReturnValue,
    IN     LPSTR   LibraryName,
    OUT    char    *Buffer,
    IN     int     Index,
    IN     int     BufLen,
    IN     BOOL    PreOrPost)
{
    int *RetVal = (int *)ReturnValue;
    SOCKET *s = va_arg(vl, SOCKET *);
    char FAR **buf = va_arg(vl, char FAR **);
    int *len = va_arg(vl, int *);
    int *flags = va_arg(vl, int *);

    wsprintf(Buffer + Index, "send() %s.\r\n",
             PreOrPost ? "->" : "<-");
    DTTextOut(DebugWindow, LogFileHandle, Buffer, OutputStyle);
    return(FALSE);
}


BOOL CALLBACK
DTHandler_sendto(
    IN     va_list vl,
    IN OUT LPVOID  ReturnValue,
    IN     LPSTR   LibraryName,
    OUT    char    *Buffer,
    IN     int     Index,
    IN     int     BufLen,
    IN     BOOL    PreOrPost)
{
    int *RetVal = (int *)ReturnValue;
    SOCKET *s = va_arg(vl, SOCKET *);
    char FAR **buf = va_arg(vl, char FAR **);
    int *len = va_arg(vl, int *);
    int *flags = va_arg(vl, int *);
    struct sockaddr FAR **to = va_arg(vl, struct sockaddr FAR **);
    int *tolen = va_arg(vl, int *);

    wsprintf(Buffer + Index, "sendto() %s.\r\n",
             PreOrPost ? "->" : "<-");
    DTTextOut(DebugWindow, LogFileHandle, Buffer, OutputStyle);
    return(FALSE);
}


BOOL CALLBACK
DTHandler_setsockopt(
    IN     va_list vl,
    IN OUT LPVOID  ReturnValue,
    IN     LPSTR   LibraryName,
    OUT    char    *Buffer,
    IN     int     Index,
    IN     int     BufLen,
    IN     BOOL    PreOrPost)
{
    int *RetVal = (int *)ReturnValue;
    SOCKET *s = va_arg(vl, SOCKET *);
    int *level = va_arg(vl, int *);
    int *optname = va_arg(vl, int *);
    char FAR **optval = va_arg(vl, char FAR **);
    int *optlen = va_arg(vl, int *);

    wsprintf(Buffer + Index, "setsockopt() %s.\r\n",
             PreOrPost ? "->" : "<-");
    DTTextOut(DebugWindow, LogFileHandle, Buffer, OutputStyle);
    return(FALSE);
}


BOOL CALLBACK
DTHandler_shutdown(
    IN     va_list vl,
    IN OUT LPVOID  ReturnValue,
    IN     LPSTR   LibraryName,
	OUT    char    *Buffer,
    IN     int     Index,
    IN     int     BufLen,
    IN     BOOL    PreOrPost)
{
    int *RetVal = (int *)ReturnValue;
    SOCKET *s = va_arg(vl, SOCKET *);
    int *how = va_arg(vl, int *);

    wsprintf(Buffer + Index, "shutdown() %s.\r\n",
             PreOrPost ? "->" : "<-");
    DTTextOut(DebugWindow, LogFileHandle, Buffer, OutputStyle);
    return(FALSE);
}


BOOL CALLBACK
DTHandler_socket(
    IN     va_list vl,
    IN OUT LPVOID  ReturnValue,
    IN     LPSTR   LibraryName,
    OUT    char    *Buffer,
    IN     int     Index,
    IN     int     BufLen,
    IN     BOOL    PreOrPost)
{
    SOCKET *RetVal = (SOCKET *)ReturnValue;
    int *af = va_arg(vl, int *);
    int *type = va_arg(vl, int *);
    int *protocol = va_arg(vl, int *);

    wsprintf(Buffer + Index, "socket() %s.\r\n",
             PreOrPost ? "->" : "<-");
    DTTextOut(DebugWindow, LogFileHandle, Buffer, OutputStyle);
    return(FALSE);
}

BOOL CALLBACK
DTHandler_gethostbyaddr(
    IN     va_list vl,
    IN OUT LPVOID  ReturnValue,
    IN     LPSTR   LibraryName,
    OUT    char    *Buffer,
    IN     int     Index,
    IN     int     BufLen,
    IN     BOOL    PreOrPost)
{
    struct hostent FAR **RetVal = (struct hostent FAR **)ReturnValue;
    char FAR **addr = va_arg(vl, char FAR **);
    int *len = va_arg(vl, int *);
    int *type = va_arg(vl, int *);

    wsprintf(Buffer + Index, "gethostbyaddr() %s.\r\n",
             PreOrPost ? "->" : "<-");
    DTTextOut(DebugWindow, LogFileHandle, Buffer, OutputStyle);
    return(FALSE);
}


BOOL CALLBACK
DTHandler_gethostbyname(
    IN     va_list vl,
    IN OUT LPVOID  ReturnValue,
    IN     LPSTR   LibraryName,
    OUT    char    *Buffer,
    IN     int     Index,
    IN     int     BufLen,
    IN     BOOL    PreOrPost)
{
    struct hostent FAR **RetVal = (struct hostent FAR **)ReturnValue;
    char FAR **name = va_arg(vl, char FAR **);

    wsprintf(Buffer + Index, "gethostbyname() %s.\r\n",
             PreOrPost ? "->" : "<-");
    DTTextOut(DebugWindow, LogFileHandle, Buffer, OutputStyle);
    return(FALSE);
}


BOOL CALLBACK
DTHandler_gethostname(
    IN     va_list vl,
    IN OUT LPVOID  ReturnValue,
    IN     LPSTR   LibraryName,
    OUT    char    *Buffer,
    IN     int     Index,
    IN     int     BufLen,
    IN     BOOL    PreOrPost)
{
    int *RetVal = (int *)ReturnValue;
    char FAR **name = va_arg(vl, char FAR **);
    int *namelen = va_arg(vl, int *);

	int off = Index;
	if (PreOrPost) {
		off += wsprintf(Buffer + off, "-> gethostname()\r\n\t");
		off += wsprintf(Buffer + off, "name: 0x%08x\r\n\t", *name);
	} else {
		off += wsprintf(Buffer + off, "<- gethostname() returned: %d\r\n\t", *RetVal);
		if (*name) {
			off += wsprintf(Buffer + off, "name: %s\r\n\t", *name);
		} else {
			off += wsprintf(Buffer + off, "name is NULL\r\n\t");
		}
	}
	wsprintf(Buffer + off, "namelen: %d (0x%x)\r\n", *namelen);
    DTTextOut(DebugWindow, LogFileHandle, Buffer, OutputStyle);

    return(FALSE);
}


BOOL CALLBACK
DTHandler_getprotobyname(
    IN     va_list vl,
    IN OUT LPVOID  ReturnValue,
    IN     LPSTR   LibraryName,
    OUT    char    *Buffer,
    IN     int     Index,
    IN     int     BufLen,
    IN     BOOL    PreOrPost)
{
    struct protoent FAR **RetVal = (struct protoent FAR **)ReturnValue;
    char FAR **name = va_arg(vl, char FAR **);

    wsprintf(Buffer + Index, "getprotobyname() %s.\r\n",
             PreOrPost ? "->" : "<-");
    DTTextOut(DebugWindow, LogFileHandle, Buffer, OutputStyle);
    return(FALSE);
}


BOOL CALLBACK
DTHandler_getprotobynumber(
    IN     va_list vl,
    IN OUT LPVOID  ReturnValue,
    IN     LPSTR   LibraryName,
    OUT    char    *Buffer,
    IN     int     Index,
    IN     int     BufLen,
    IN     BOOL    PreOrPost)
{
    struct protoent FAR **RetVal = (struct protoent FAR **)ReturnValue;
    int *number = va_arg(vl, int *);

    wsprintf(Buffer + Index, "getprotobynumber() %s.\r\n",
             PreOrPost ? "->" : "<-");
    DTTextOut(DebugWindow, LogFileHandle, Buffer, OutputStyle);
    return(FALSE);
}


BOOL CALLBACK
DTHandler_getservbyname(
    IN     va_list vl,
    IN OUT LPVOID  ReturnValue,
    IN     LPSTR   LibraryName,
    OUT    char    *Buffer,
    IN     int     Index,
    IN     int     BufLen,
    IN     BOOL    PreOrPost)
{
    struct servent FAR **RetVal = (struct servent FAR **)ReturnValue;
    char FAR **name = va_arg(vl, char FAR **);
    char FAR **proto = va_arg(vl, char FAR **);

    wsprintf(Buffer + Index, "getservbyname() %s.\r\n",
             PreOrPost ? "->" : "<-");
    DTTextOut(DebugWindow, LogFileHandle, Buffer, OutputStyle);
    return(FALSE);
}


BOOL CALLBACK
DTHandler_getservbyport(
    IN     va_list vl,
    IN OUT LPVOID  ReturnValue,
    IN     LPSTR   LibraryName,
    OUT    char    *Buffer,
    IN     int     Index,
    IN     int     BufLen,
    IN     BOOL    PreOrPost)
{
    struct servent FAR **RetVal = (struct servent FAR **)ReturnValue;
    int *port = va_arg(vl, int *);
    char FAR **proto = va_arg(vl, char FAR **);

    wsprintf(Buffer + Index, "getservbyport() %s.\r\n",
             PreOrPost ? "->" : "<-");
    DTTextOut(DebugWindow, LogFileHandle, Buffer, OutputStyle);
    return(FALSE);
}


BOOL CALLBACK
DTHandler_inet_addr(
    IN     va_list vl,
    IN OUT LPVOID  ReturnValue,
    IN     LPSTR   LibraryName,
    OUT    char    *Buffer,
    IN     int     Index,
    IN     int     BufLen,
    IN     BOOL    PreOrPost)
{
    unsigned long *RetVal = (unsigned long *)ReturnValue;
    char FAR **cp = va_arg(vl, char FAR **);

	int off = Index;
	if (PreOrPost) {
		off += wsprintf(Buffer + off, "-> inet_addr()\r\n\t");
		if (*cp) {
			off += wsprintf(Buffer + off, "cp: %s\r\n", *cp);
		} else {
			off += wsprintf(Buffer + off, "cp is NULL\r\n");
		}
	} else {
		off += wsprintf(Buffer + off, 
			"<- inet_addr() returned: 0x%0x08x\r\n", *RetVal);
	}
    DTTextOut(DebugWindow, LogFileHandle, Buffer, OutputStyle);
    return(FALSE);
}


BOOL CALLBACK
DTHandler_inet_ntoa(
    IN     va_list vl,
    IN OUT LPVOID  ReturnValue,
    IN     LPSTR   LibraryName,
    OUT    char    *Buffer,
    IN     int     Index,
    IN     int     BufLen,
    IN     BOOL    PreOrPost)
{
    char FAR **RetVal = (char FAR **)ReturnValue;
    struct in_addr *in = va_arg(vl, struct in_addr *);

	int off = Index;
	if (PreOrPost) {
		off += wsprintf(Buffer + off, "-> inet_ntoa()\r\n\t");
		FormatInAddr(in, Buffer, off);
	} else {
		wsprintf(Buffer + off, 
			"<- inet_addr() returned: %s\r\n", *RetVal);
	}
    DTTextOut(DebugWindow, LogFileHandle, Buffer, OutputStyle);
    return(FALSE);
}
